/*
 *******************************************************
  
 cat an mice benchmark, wodes08, libfaudes 2.09

 this file preforms the monolithic supervisor synthesis for the 
 cat-and-mouse benchmark example, where individual maze levels are modeled.
 
 *******************************************************
 */ 



#include <stdio.h>
#include <iostream>
#include "libfaudes.h"
#include "catsMice_levels_models.h"

using namespace std;
using namespace faudes;

// MonolithicCat_and_Mouse(n, k)
void MonolithicCat_and_Mouse(const int n, const int k){
  // helper data structures for intermediate computations 
  cGenerator maze, mazeSpec,sup, tmpPlant, tmpSpec, plant, spec;
  std::string nstr, kstr;
  nstr = ToStringInteger(n);
  kstr = ToStringInteger(k);
  // dummy high-level alphabet
  EventSet highAlphabet;
  // all maze levels (plant and specification) are constructed and composed by the parallel composition
  for(int l = 1; l <= n; l++){
    // computation of the model for the current maze level
    Maze(maze,mazeSpec,n,k,l,highAlphabet);
    // compose the current maze level model with the existing models
    if(l == 1){
      plant = maze;
      spec = mazeSpec;
    }
    else if(l % 2 == 0){
      cParallel(plant,maze,tmpPlant);
      cParallel(spec,mazeSpec,tmpSpec);
      plant.Clear();
      spec.Clear();
    }
    else{
      cParallel(tmpSpec,mazeSpec,spec);
      cParallel(tmpPlant,maze,plant);
      tmpPlant.Clear();
      tmpSpec.Clear();
    }  
  }
  // perform monolithic supervisor synthesis
  std::cout << "#############################################################\n";
  std::cout << "#Result of Monolithic Synthesis for n = " << n << " and k = " << k << endl;
  if(n % 2 == 1){
      std::cout << "plant size: " << plant.Size() << ", transitions: " << plant.TransRel().Size() 
           << ", spec size: " << spec.Size() << ", transitions: " << spec.TransRel().Size(); 
      cSupConNB(plant,spec,sup);
  }
  else{
      std::cout << "plant size: " << tmpPlant.Size() << ", transitions: " << tmpPlant.TransRel().Size() 
           << ", spec size: " << tmpSpec.Size() << ", transitions: " << tmpSpec.TransRel().Size(); 
   
      cSupConNB(tmpPlant,tmpSpec,sup);

  }
  sup.ClearStateNames();
  //sup.Write( ("./cat_and_mouse_Data/monolithicSup("+nstr+","+kstr+").gen").c_str() );
  std::cout  << ", closed loop size: " << sup.Size() << ", transitions: " << sup.TransRel().Size() << endl; 
  std::cout << "#############################################################\n";
  sup.Write("./results/monolithic/sup.gen");
}



 
// Main function for the monolithic synthesis
int main(int argc, char* argv[]) {
    try{
        // command line parsing; two input parameters have to be provided
        if(argc!=3) {
            cerr<<"usage: ./catsMice_individual_monolithic <n> <k> "<<endl;
            return(-1);
        }
        // number of levels
        int n=atoi(argv[1]);
        // number of cats and mice
        int k=atoi(argv[2]);
        // Perform the monolithic supervisor synthesis
        MonolithicCat_and_Mouse(n,k);
    } //endof try
    catch (Exception& e) {
        std::cout << "function: " << e.Where() << std::endl;
        std::cout << "exception descrition: " << e.What() << std::endl;
        std::cout << "exception id: " << e.Id() << std::endl;
        return(-1);
    }
    // return 0 after correct program execution
    return 0;
}
