/** @file op_ex_mc.cpp 

This tutorial demonstrates methods for verifying the mutual controllability condition of 
two given automata. A definition of mutual controllability is given in 
S.-H. Lee and K. C. Wong, “Structural decentralised control of concurrent
DES,” European Journal of Control, vol. 35, pp. 1125-1134,2002.

@ingroup Tutorials

@include op_ex_mc.cpp
*/

#include <stdio.h>
#include <iostream>
#include <libfaudes.h>
#include <op_observer.h>



int main(int argc, char* argv[])
{

	// read two generators from file input
	cGenerator gen1("./data/ex_mc/ex_mc1.gen");
	cGenerator gen2("./data/ex_mc/ex_mc2.gen");
	
	// Verify mutual controllability of gen1 and gen2. In this case, mutual controllabiity is violated.
	bool mc = MutualControllability(gen1,gen2);
	std::cout << "#################################################\n";
	std::cout << "# Mutual controllability verification - result: " << mc << endl;
	std::cout << "#################################################\n";
	
	// read one further generator from file input
	cGenerator gen3("./data/ex_mc/ex_mc3.gen");
	// Verify mutual controllability of gen3 and gen2. In this case, mutual controllability holds.
	mc = MutualControllability(gen3,gen2);
	std::cout << "##################################################\n";
	std::cout << "# Mutual controllability verification - result: " << mc << endl;
	std::cout << "##################################################\n";

  	
return 0;
}
