/* luafaudes.cpp - registers FAUDES funcitionality in Lua */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2006 Bernd Opitz
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */


#include "luafaudes.hpp"

int luafaudes_Help(lua_State* L) {
	std::stringstream sstr;
	sstr 
		<< std::endl
		<< "faudes Bindings:" << std::endl
		<< std::endl
		<< "  HelpFunctions()             print function list" << std::endl
		<< "  HelpGenerator()             print Generator methods" << std::endl
		<< "  HelpEventSet()              print EventSet methods" << std::endl
#ifdef USING_PLUGIN_OBSERVER
		<< "  HelpObserver()              print observer-plugin methods" << std::endl
#endif
		<< std::endl
		<< "faudes Configuration:" << std::endl
		<< std::endl
		<< "  StatenamesOn()              enable automatic state names" << std::endl
		<< "  StatenamesOff()             disnable automatic state names" << std::endl
		<< "  DotExecPath(\"filename\")     path of dot executable" << std::endl
		<< "  Version()                   report libfaudes version string" << std::endl
		<< std::endl
		<< "luafaudes Console Commands:" << std::endl
		<< std::endl
		<< "  InitLog(\"filename\")         init logging of commands" << std::endl
		<< "  CloseLog()                  finish logging of commands" << std::endl
		<< "  Ctrl-C                      exit program" << std::endl;
	DisplayText("luafaudes Help", sstr.str());
	return 0;
}

void register_luafaudes(lua_State* L) {
	// register console helper functions
	RegisterConsoleFunctions(L);

	// register common functions
	RegisterCommonFaudesFunctions(L);

	// register Generator class as Generator userdata type
	RegisterGeneratorClass(L);

	// register Generator userdata functions
	RegisterGeneratorFunctions(L);


	// register EventSet class as EventSet userdata type
	RegisterEventSetClass(L);

#ifdef USING_PLUGIN_OBSERVER
	RegisterObserverFunctions(L);
#endif

	// register help function
	lua_register(L, "Help", luafaudes_Help);
}
